import ball
import cube
import ellipse
import lshape
import lshape3d
import pacman
import rectangle
import rectangle_with_hole
import tetrahedron
import torus

__all__ = [
    "ball",
    "cube",
    "ellipse",
    "lshape",
    "lshape3d",
    "pacman",
    "rectangle",
    "rectangle_with_hole",
    "tetrahedron",
    "torus",
]
